#include "stdafx.h"
#include "EMFDecode.h"


/****************************************************************************
*                              EMFDecode::Decode
* Inputs:
*       const ENHMETARECORD * rec:
*       LPVOID p1:
*       LPVOID p2:
* Result: void
*       
* Effect: 
*       Decodes an element
****************************************************************************/

void EMFDecode::Decode(const ENHMETARECORD * rec, LPVOID p1, LPVOID p2)
    {
#define DECODE_CASE(x, fn) case x: fn(rec, p1, p2); break;
     switch(rec->iType)
        { /* type */
         DECODE_CASE(EMR_ABORTPATH, DecodeAbortPath);
         DECODE_CASE(EMR_ALPHABLEND, DecodeAlphaBlend);
         DECODE_CASE(EMR_ANGLEARC, DecodeAngleArc);
         DECODE_CASE(EMR_ARC, DecodeArc);
         DECODE_CASE(EMR_ARCTO, DecodeArcTo);
         DECODE_CASE(EMR_BEGINPATH, DecodeBeginPath);
         DECODE_CASE(EMR_BITBLT, DecodeBitBlt);
         DECODE_CASE(EMR_CHORD, DecodeChord);
         DECODE_CASE(EMR_CLOSEFIGURE, DecodeCloseFigure);
         DECODE_CASE(EMR_COLORCORRECTPALETTE, DecodeColorCorrectPalette);
         DECODE_CASE(EMR_COLORMATCHTOTARGETW, DecodeColorMatchToTargetW);
         DECODE_CASE(EMR_CREATEBRUSHINDIRECT, DecodeCreateBrushIndirect);
         DECODE_CASE(EMR_CREATECOLORSPACE, DecodeCreateColorSpace);
         DECODE_CASE(EMR_CREATECOLORSPACEW, DecodeCreateColorSpaceW);
         DECODE_CASE(EMR_CREATEDIBPATTERNBRUSHPT, DecodeCreateDIBPatternBrushPt);
         DECODE_CASE(EMR_CREATEMONOBRUSH, DecodeCreateMonoBrush);
         DECODE_CASE(EMR_CREATEPALETTE, DecodeCreatePalette);
         DECODE_CASE(EMR_CREATEPEN, DecodeCreatePen);
         DECODE_CASE(EMR_DELETECOLORSPACE, DecodeDeleteColorSpace);
         DECODE_CASE(EMR_DELETEOBJECT, DecodeDeleteObject);
         DECODE_CASE(EMR_ELLIPSE, DecodeEllipse);
         DECODE_CASE(EMR_ENDPATH, DecodeEndPath);
         DECODE_CASE(EMR_EOF, DecodeEOF);
         DECODE_CASE(EMR_EXCLUDECLIPRECT, DecodeExcludeClipRect);
         DECODE_CASE(EMR_EXTCREATEFONTINDIRECTW, DecodeExtCreateFontIndirectW);
         DECODE_CASE(EMR_EXTCREATEPEN, DecodeExtCreatePen);
         DECODE_CASE(EMR_EXTFLOODFILL, DecodeExtFloodFill);
         DECODE_CASE(EMR_EXTSELECTCLIPRGN, DecodeExtSelectClipRgn);
         DECODE_CASE(EMR_EXTTEXTOUTA, DecodeExtTextOutA);
         DECODE_CASE(EMR_EXTTEXTOUTW, DecodeExtTextOutW);
         DECODE_CASE(EMR_FILLPATH, DecodeFillPath);
         DECODE_CASE(EMR_FILLRGN, DecodeFillRgn);
         DECODE_CASE(EMR_FLATTENPATH, DecodeFlattenPath);
         DECODE_CASE(EMR_FRAMERGN, DecodeFrameRgn);
         DECODE_CASE(EMR_GDICOMMENT, DecodeGdiComment);
         DECODE_CASE(EMR_GLSBOUNDEDRECORD, DecodeGLSBoundedRecord);
         DECODE_CASE(EMR_GLSRECORD, DecodeGLSRecord);
         DECODE_CASE(EMR_GRADIENTFILL, DecodeGradientFill);
         DECODE_CASE(EMR_HEADER, DecodeHeader);
         DECODE_CASE(EMR_INTERSECTCLIPRECT, DecodeIntersectClipRect);
         DECODE_CASE(EMR_INVERTRGN, DecodeInvertRgn);
         DECODE_CASE(EMR_LINETO, DecodeLineTo);
         DECODE_CASE(EMR_MASKBLT, DecodeMaskBlt);
         DECODE_CASE(EMR_MODIFYWORLDTRANSFORM, DecodeModifyWorldTransform);
         DECODE_CASE(EMR_MOVETOEX, DecodeMoveToEx);
         DECODE_CASE(EMR_OFFSETCLIPRGN, DecodeOffsetClipRgn);
         DECODE_CASE(EMR_PAINTRGN, DecodePaintRgn);
         DECODE_CASE(EMR_PIE, DecodePie);
         DECODE_CASE(EMR_PIXELFORMAT, DecodePixelFormat);
         DECODE_CASE(EMR_PLGBLT, DecodePlgBlt);
         DECODE_CASE(EMR_POLYBEZIER, DecodePolyBezier);
         DECODE_CASE(EMR_POLYBEZIER16, DecodePolyBezier16);
         DECODE_CASE(EMR_POLYBEZIERTO, DecodePolyBezierTo);
         DECODE_CASE(EMR_POLYBEZIERTO16, DecodePolyBezierTo16);
         DECODE_CASE(EMR_POLYDRAW, DecodePolyDraw);
         DECODE_CASE(EMR_POLYDRAW16, DecodePolyDraw16);
         DECODE_CASE(EMR_POLYGON, DecodePolygon);
         DECODE_CASE(EMR_POLYGON16, DecodePolygon16);
         DECODE_CASE(EMR_POLYLINE, DecodePolyLine);
         DECODE_CASE(EMR_POLYLINE16, DecodePolyLine16);
         DECODE_CASE(EMR_POLYLINETO, DecodePolyLineTo);
         DECODE_CASE(EMR_POLYLINETO16, DecodePolyLineTo16);
         DECODE_CASE(EMR_POLYPOLYGON, DecodePolyPolygon);
         DECODE_CASE(EMR_POLYPOLYGON16, DecodePolyPolygon16);
         DECODE_CASE(EMR_POLYPOLYLINE, DecodePolyPolyLine);
         DECODE_CASE(EMR_POLYPOLYLINE16, DecodePolyPolyLine16);
         DECODE_CASE(EMR_POLYTEXTOUTA, DecodePolyTextOutA);
         DECODE_CASE(EMR_POLYTEXTOUTW, DecodePolyTextOutW);
         DECODE_CASE(EMR_REALIZEPALETTE, DecodeRealizePalette);
         DECODE_CASE(EMR_RECTANGLE, DecodeRectangle);
         DECODE_CASE(EMR_RESIZEPALETTE, DecodeResizePalette);
         DECODE_CASE(EMR_RESTOREDC, DecodeRestoreDC);
         DECODE_CASE(EMR_ROUNDRECT, DecodeRoundRect);
         DECODE_CASE(EMR_SAVEDC, DecodeSaveDC);
         DECODE_CASE(EMR_SCALEVIEWPORTEXTEX, DecodeScaleViewportExtEx);
         DECODE_CASE(EMR_SCALEWINDOWEXTEX, DecodeScaleWindowExtEx);
         DECODE_CASE(EMR_SELECTCLIPPATH, DecodeSelectClipPath);
         //DECODE_CASE(EMR_SELECTCOLORSPACE, DecodeSelectColorSpace);
         DECODE_CASE(EMR_SELECTOBJECT, DecodeSelectObject);
         DECODE_CASE(EMR_SELECTPALETTE, DecodeSelectPalette);
         DECODE_CASE(EMR_SETARCDIRECTION, DecodeSetArcDirection);
         DECODE_CASE(EMR_SETBKCOLOR, DecodeSetBkColor);
         DECODE_CASE(EMR_SETBKMODE, DecodeSetBkMode);
         DECODE_CASE(EMR_SETBRUSHORGEX, DecodeSetBrushOrgEx);
         DECODE_CASE(EMR_SETCOLORADJUSTMENT, DecodeSetColorAdjustment);
         DECODE_CASE(EMR_SETCOLORSPACE, DecodeSetColorSpace);
         DECODE_CASE(EMR_SETICMMODE, DecodeSetICMMode);
         DECODE_CASE(EMR_SETICMPROFILEA, DecodeSetICMProfileA);
         DECODE_CASE(EMR_SETICMPROFILEW, DecodeSetICMProfileW);
         DECODE_CASE(EMR_SETLAYOUT, DecodeSetLayout);
         DECODE_CASE(EMR_SETMAPMODE, DecodeSetMapMode);
         DECODE_CASE(EMR_SETMAPPERFLAGS, DecodeSetMapperFlags);
         DECODE_CASE(EMR_SETMETARGN, DecodeSetMetaRegion);
         DECODE_CASE(EMR_SETMITERLIMIT, DecodeSetMiterLimit);
         DECODE_CASE(EMR_SETPALETTEENTRIES, DecodeSetPaletteEntries);
         DECODE_CASE(EMR_SETPIXELV, DecodeSetPixelV);
         DECODE_CASE(EMR_SETPOLYFILLMODE, DecodeSetPolyFillMode);
         DECODE_CASE(EMR_SETROP2, DecodeSetROP2);
         DECODE_CASE(EMR_SETSTRETCHBLTMODE, DecodeSetStretchBltMode);
         DECODE_CASE(EMR_SETTEXTALIGN, DecodeSetTextAlign);
         DECODE_CASE(EMR_SETTEXTCOLOR, DecodeSetTextColor);
         DECODE_CASE(EMR_SETVIEWPORTEXTEX, DecodeSetViewportExtEx);
         DECODE_CASE(EMR_SETVIEWPORTORGEX, DecodeSetViewportOrgEx);
         DECODE_CASE(EMR_SETWINDOWEXTEX, DecodeSetWindowExtEx);
         DECODE_CASE(EMR_SETWINDOWORGEX, DecodeSetWindowOrgEx);
         DECODE_CASE(EMR_SETWORLDTRANSFORM, DecodeSetWorldTransform);
         DECODE_CASE(EMR_STRETCHBLT, DecodeStretchBlt);
         DECODE_CASE(EMR_STRETCHDIBITS, DecodeStretchDIBits);
         DECODE_CASE(EMR_STROKEANDFILLPATH, DecodeStrokeAndFillPath);
         DECODE_CASE(EMR_STROKEPATH, DecodeStrokePath);
         DECODE_CASE(EMR_TRANSPARENTBLT, DecodeTransparentBlt);
         DECODE_CASE(EMR_WIDENPATH, DecodeWidenPath);
        } /* type */
    } // EMFDecode::Decode
